@@ -399,7 +399,7 @@ class VariableClPkgdirPath(Variable):
                 self.Get('os_install_linux_shortname'),
                 self.Get('os_install_arch_machine'))
 
-class VariableOsInstallDevFrom(Variable):
+class VariableClInstallDevFrom(Variable):
     """
     Root device of previous installed os
     """
@@ -725,18 +725,21 @@ class PackageCheckHelper(ReadonlyVariable):
     image = False
     package = None
     type = "bool"
+    image_variable = "cl_image"
+    prefix_variable = "cl_chroot_path"
 
     def get(self):
         try:
             if self.image:
-                image = self.Get('cl_image')
+                image = self.Get(self.image_variable)
                 if image:
                     with image as distr:
                         distrPath = image.getDirectory()
                         if isPkgInstalled(self.package,prefix=distrPath):
                             return "on"
             else:
-                if isPkgInstalled(self.package):
+                prefix = self.Get(self.prefix_variable)
+                if isPkgInstalled(self.package, prefix=prefix):
                     return "on"
         except:
             pass
